package cn.estsh.i3plus.ext.mes.pcn.apiservice.daoimpl;


import cn.estsh.i3plus.ext.mes.pcn.apiservice.dao.IMesEquipmentLogDao;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentLog;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class MesEquipmentLogDao implements IMesEquipmentLogDao {

    @Value("${mes.sharding.append.org:}")
    public String mesShardingAppendOrg;

    @Autowired
    private EntityManager entityManager;

    @Override
    public List<MesEquipVariableCollectContext> queryMesEquipmentLog(List<MesEquipmentVariable> equipmentVariableList, String organizeCode, Integer equipId, Integer variableType) {

        List<MesEquipVariableCollectContext> equipVariableCollectContextList = generateEquipVariableCollectContextList(equipmentVariableList);

        if (CollectionUtils.isEmpty(equipmentVariableList) || StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || StringUtils.isEmpty(variableType)) return null;

        StringBuffer builder = new StringBuffer();

        builder.append(" select e.equip_variable_id, e.equip_variable_value, e.equip_variable_status, e.quality, e.modify_date_time, e.id ");
        builder.append(" from mes_equipment_log_");
        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
        builder.append(equipId);
        builder.append(" e where e.organize_code = :organizeCode and e.variable_type = :variableType ");

        Query query = entityManager.createNativeQuery(builder.toString());
        query.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesPcnExtConstWords.VARIABLE_TYPE, variableType);
        List list = query.getResultList();

        if (CollectionUtils.isEmpty(list)) return equipVariableCollectContextList;

        List<String> equipVariableIdList = equipmentVariableList.stream().filter(o -> null != o).map(o -> o.getId().toString()).collect(Collectors.toList());

        for (Object equipmentLog : list) {

            if (null == equipmentLog) continue;

            Object[] equipmentLogArr = (Object[]) equipmentLog;

            if (null == equipmentLogArr || equipmentLogArr.length != 6 || StringUtils.isEmpty(equipmentLogArr[0]) || !equipVariableIdList.contains(equipmentLogArr[0].toString())) continue;

            equipVariableCollectContextList.stream().filter(o -> (null != o && o.getEquipVariableId().toString().equals(equipmentLogArr[0].toString()))).findFirst().get().copyValue(equipmentLogArr[1], equipmentLogArr[2], equipmentLogArr[3], equipmentLogArr[4], equipmentLogArr[5]);

        }

        return equipVariableCollectContextList;
    }

    @Override
    public void updateEquipVariableStatus(String organizeCode, Integer equipId, Map<Long, String> equipmentLogIdMap, Integer equipVariableStatus) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || CollectionUtils.isEmpty(equipmentLogIdMap) || StringUtils.isEmpty(equipVariableStatus)) return;

        StringBuffer builder = new StringBuffer();

        for (int i = 0; i < equipmentLogIdMap.size(); i ++) {

            builder.append(" update mes_equipment_log_");
            if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
            builder.append(equipId);
            builder.append(" set equip_variable_status = :equipVariableStatus_");
            builder.append(i);
            builder.append(" where id = :id_");
            builder.append(i);
            builder.append(" and modify_date_time = :modifyDatetime_");
            builder.append(i);
            builder.append(" ;");

        }

        Query update = entityManager.createNativeQuery(builder.toString());

        Integer index = 0;
        for (Map.Entry<Long, String> entry : equipmentLogIdMap.entrySet()) {
            if (null == entry) continue;
            update.setParameter(String.format("%s_%s", MesPcnExtConstWords.EQUIP_VARIABLE_STATUS, index), equipVariableStatus);
            update.setParameter(String.format("%s_%s", MesPcnExtConstWords.ID, index), entry.getKey());
            update.setParameter(String.format("%s_%s", MesPcnExtConstWords.MODIFY_DATE_TIME, index), entry.getValue());
            index ++;
        }

        update.executeUpdate();

    }

    private List<MesEquipVariableCollectContext> generateEquipVariableCollectContextList(List<MesEquipmentVariable> equipmentVariableList) {
        List<MesEquipVariableCollectContext> equipVariableCollectContextList = new ArrayList<>();
        if (CollectionUtils.isEmpty(equipmentVariableList)) return equipVariableCollectContextList;
        equipmentVariableList.stream().filter(o -> null != o).forEach(o -> {
            MesEquipVariableCollectContext equipVariableCollectContext = new MesEquipVariableCollectContext(o.getId());
            BeanUtils.copyProperties(o, equipVariableCollectContext);
            equipVariableCollectContextList.add(equipVariableCollectContext);
        });
        return equipVariableCollectContextList;
    }

    @Override
    public List<MesEquipVariableCollectContext> queryMesEquipmentLog(String organizeCode, Integer equipId, Integer variableType, List<Long> equipVariableIdList) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || StringUtils.isEmpty(variableType) || CollectionUtils.isEmpty(equipVariableIdList)) return null;

        equipVariableIdList = equipVariableIdList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        StringBuffer builder = new StringBuffer();

        builder.append(" select e.equip_variable_id, e.equip_variable_value, e.equip_variable_status, e.quality, e.modify_date_time, e.id ");
        builder.append(" from mes_equipment_log_");
        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
        builder.append(equipId);
        builder.append(" e where e.organize_code = :organizeCode and e.variable_type = :variableType ");
        if (equipVariableIdList.size() == 1) builder.append(" and e.equip_variable_id = :equipVariableId ");
        else builder.append(" and e.equip_variable_id in (:equipVariableId) ");

        Query query = entityManager.createNativeQuery(builder.toString());
        query.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesPcnExtConstWords.VARIABLE_TYPE, variableType);
        if (equipVariableIdList.size() == 1) query.setParameter(MesPcnExtConstWords.EQUIP_VARIABLE_ID, equipVariableIdList.get(0));
        else query.setParameter(MesPcnExtConstWords.EQUIP_VARIABLE_ID, equipVariableIdList);
        List list = query.getResultList();

        return backEquipVariableCollectContextList(list);
    }

    @Override
    public List<MesEquipVariableCollectContext> queryMesEquipmentLog(String organizeCode, Integer equipId, Integer variableType) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipId) || StringUtils.isEmpty(variableType)) return null;

        StringBuffer builder = new StringBuffer();

        builder.append(" select e.equip_variable_id, e.equip_variable_value, e.equip_variable_status, e.quality, e.modify_date_time, e.id ");
        builder.append(" from mes_equipment_log_");
        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(organizeCode.toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
        builder.append(equipId);
        builder.append(" e where e.organize_code = :organizeCode and e.variable_type = :variableType ");

        Query query = entityManager.createNativeQuery(builder.toString());
        query.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesPcnExtConstWords.VARIABLE_TYPE, variableType);
        List list = query.getResultList();

        return backEquipVariableCollectContextList(list);

    }

    private List<MesEquipVariableCollectContext> backEquipVariableCollectContextList(List list) {

        if (CollectionUtils.isEmpty(list)) return null;

        List<MesEquipVariableCollectContext> equipVariableCollectContextList = new ArrayList<>();

        for (Object equipmentLog : list) {

            if (null == equipmentLog) continue;

            Object[] equipmentLogArr = (Object[]) equipmentLog;

            if (null == equipmentLogArr || equipmentLogArr.length != 6 || StringUtils.isEmpty(equipmentLogArr[0])) continue;

            equipVariableCollectContextList.add(new MesEquipVariableCollectContext(Long.valueOf(equipmentLogArr[0].toString())).copyValue(equipmentLogArr[1], equipmentLogArr[2], equipmentLogArr[3], equipmentLogArr[4], equipmentLogArr[5]));

        }

        return equipVariableCollectContextList;
    }

    @Override
    public void insertBatchEquipmentLogDetail(String organizeCode, Integer equipId, String userInfo, MesEquipmentRecord mesEquipmentRecord, List<MesEquipmentLog> equipmentLogList) {

        StringBuffer builder = new StringBuffer();

        for (MesEquipmentLog equipmentLog : equipmentLogList) {

            if (null == equipmentLog) continue;

            


        }
























//        StringBuffer builder = new StringBuffer();
//
//        builder.append(" insert into mes_work_cell_scan_monitor_log_");
//        if (!StringUtils.isEmpty(mesShardingAppendOrg) && mesShardingAppendOrg.toUpperCase().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.name())) builder.append(resultBean.getOrganizeCode().toLowerCase()).append(MesPcnExtConstWords.E_UNDERLINE);
//        builder.append(resultBean.getWorkCenterCode().toLowerCase());
//        builder.append("( id, organize_code, is_valid, is_deleted, create_user, create_date_time, modify_user, modify_date_time,");
//        builder.append(" description, remark, system_sync_date_time, system_sync_status,");
//        builder.append(" area_code, work_center_code, work_cell_code, equipment_code, equipment_name, process_code, process_name, craft_code, craft_name,");
//        builder.append(" mould_record_id, scan_info, work_order_no, serial_number, product_sn, cust_sn, part_no, part_name,");
//        builder.append(" step_code, log_type, message, message_type, deal_status, org_work_code)");
//        builder.append(" values ( :id , :organizeCode , :isValid , :isDeleted , :createUser , :createDatetime , :modifyUser , :modifyDatetime ,");
//        builder.append(" :description , :remark , :systemSyncDatetime , :systemSyncStatus ,");
//        builder.append(" :areaCode , :workCenterCode , :workCellCode , :equipmentCode , :equipmentName , :processCode , :processName , :craftCode , :craftName ,");
//        builder.append(" :mouldRecordId , :scanInfo , :workOrderNo , :serialNumber , :productSn , :custSn , :partNo , :partName ,");
//        builder.append(" :stepCode , :logType , :message , :messageType , :dealStatus , :orgWorkCode )");
//
//        Query insert = entityManager.createNativeQuery(builder.toString());
//
//        insert.setParameter(MesPcnExtConstWords.ID, snowflakeIdMaker.nextId());
//        insert.setParameter(MesPcnExtConstWords.ORGANIZE_CODE, resultBean.getOrganizeCode());
//        insert.setParameter(MesPcnExtConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
//        insert.setParameter(MesPcnExtConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
//        insert.setParameter(MesPcnExtConstWords.CREATE_USER, resultBean.getUserInfo());
//        insert.setParameter(MesPcnExtConstWords.CREATE_DATE_TIME, TimeTool.getNowTime(true));
//        insert.setParameter(MesPcnExtConstWords.MODIFY_USER, resultBean.getUserInfo());
//        insert.setParameter(MesPcnExtConstWords.MODIFY_DATE_TIME, TimeTool.getNowTime(true));
//        insert.setParameter(MesPcnExtConstWords.DESCRIPTION, null);
//        insert.setParameter(MesPcnExtConstWords.REMARK, null);
//        insert.setParameter(MesPcnExtConstWords.SYSTEM_SYNC_DATE_TIME, null);
//        insert.setParameter(MesPcnExtConstWords.AREA_CODE, resultBean.getAreaCode());
//        insert.setParameter(MesPcnExtConstWords.WORK_CENTER_CODE, resultBean.getWorkCenterCode());
//        insert.setParameter(MesPcnExtConstWords.WORK_CELL_CODE, resultBean.getWorkCellCode());
//        insert.setParameter(MesPcnExtConstWords.EQUIPMENT_CODE, scanMonitorContext.getEquipmentCode());
//        insert.setParameter(MesPcnExtConstWords.EQUIPMENT_NAME, scanMonitorContext.getEquipmentName());
//        insert.setParameter(MesPcnExtConstWords.PROCESS_CODE, scanMonitorContext.getProcessCode());
//        insert.setParameter(MesPcnExtConstWords.PROCESS_NAME, scanMonitorContext.getProcessName());
//        insert.setParameter(MesPcnExtConstWords.CRAFT_CODE, scanMonitorContext.getCraftCode());
//        insert.setParameter(MesPcnExtConstWords.CRAFT_NAME, scanMonitorContext.getCraftName());
//        insert.setParameter(MesPcnExtConstWords.MOULD_RECORD_ID, scanMonitorContext.getMouldRecordId());
//        insert.setParameter(MesPcnExtConstWords.SCAN_INFO, resultBean.getScanInfo());
//        insert.setParameter(MesPcnExtConstWords.WORK_ORDER_NO, null);
//        insert.setParameter(MesPcnExtConstWords.SERIAL_NUMBER, null);
//        insert.setParameter(MesPcnExtConstWords.PRODUCT_SN, null);
//        insert.setParameter(MesPcnExtConstWords.CUST_SN, null);
//        insert.setParameter(MesPcnExtConstWords.PART_NO, null);
//        insert.setParameter(MesPcnExtConstWords.PART_NAME, null);
//        insert.setParameter(MesPcnExtConstWords.STEP_CODE, resultBean.getStepCode());
//        insert.setParameter(MesPcnExtConstWords.LOG_TYPE, resultBean.getLogType());
//        insert.setParameter(MesPcnExtConstWords.MESSAGE, resultBean.getMessage());
//        insert.setParameter(MesPcnExtConstWords.MESSAGE_TYPE, MesPcnEnumUtil.WORK_CELL_SCAN_MONITOR_MESSAGE_TYPE.codeOfValue(resultBean.getDataType()));
//        insert.setParameter(MesPcnExtConstWords.ORG_WORK_CODE, new StringJoiner(MesPcnExtConstWords.E_UNDERLINE).add(resultBean.getOrganizeCode()).add(resultBean.getWorkCenterCode()).toString());
//        //存在开模ID的记录延迟同步
//        if (!StringUtils.isEmpty(scanMonitorContext.getMouldRecordId())) {
//            insert.setParameter(MesPcnExtConstWords.SYSTEM_SYNC_STATUS, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
//            insert.setParameter(MesPcnExtConstWords.DEAL_STATUS, MesExtEnumUtil.MES_LOG_DEAL_STATUS.UNDEAL.getValue());
//        } else {
//            insert.setParameter(MesPcnExtConstWords.SYSTEM_SYNC_STATUS, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
//            insert.setParameter(MesPcnExtConstWords.DEAL_STATUS, MesExtEnumUtil.MES_LOG_DEAL_STATUS.DEAL_SUCCESS.getValue());
//        }
//
//        insert.executeUpdate();


    }

}
